@@ -1,7 +1,11 @@
+#include "config.h"
+
 #include <fcntl.h>
 #include <limits.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <stdarg.h>
+#include <string.h>
 #include <unistd.h>
 #include <getopt.h>
 #include <dirent.h>
@@ -13,10 +17,7 @@
 #include <sys/wait.h>
 #include <sys/file.h>
 
-#include "version.h"
-#include "kbd_error.h"
-#include "xmalloc.h"
-#include "getfd.h"
+#include "libcommon.h"
 
 #ifdef COMPAT_HEADERS
 #include "compat/linux-limits.h"
@@ -27,7 +28,7 @@
 #define FALSE (0)
 
 #ifdef ESIX_5_3_2_D
-#define	VTBASE "/dev/vt%02d"
+#define VTBASE "/dev/vt%02d"
 #endif
 
 // Where your VTs are hidden
@@ -41,25 +42,26 @@
 #endif
 
 static void
-__attribute__ ((noreturn))
-print_help(int ret)
+    __attribute__((noreturn))
+    print_help(int ret)
 {
 	printf(_("Usage: %s [OPTIONS] -- command\n"
-	       "\n"
-	       "This utility help you to start a program on a new virtual terminal (VT).\n"
-	       "\n"
-	       "Options:\n"
-	       "  -c, --console=NUM   use the given VT number;\n"
-	       "  -e, --exec          execute the command, without forking;\n"
-	       "  -f, --force         force opening a VT without checking;\n"
-	       "  -l, --login         make the command a login shell;\n"
-	       "  -u, --user          figure out the owner of the current VT;\n"
-	       "  -s, --switch        switch to the new VT;\n"
-	       "  -w, --wait          wait for command to complete;\n"
-	       "  -v, --verbose       print a message for each action;\n"
-	       "  -V, --version       print program version and exit;\n"
-	       "  -h, --help          output a brief help message.\n"
-	       "\n"), progname);
+	         "\n"
+	         "This utility helps you to start a program on a new virtual terminal (VT).\n"
+	         "\n"
+	         "Options:\n"
+	         "  -c, --console=NUM   use the given VT number;\n"
+	         "  -e, --exec          execute the command, without forking;\n"
+	         "  -f, --force         force opening a VT without checking;\n"
+	         "  -l, --login         make the command a login shell;\n"
+	         "  -u, --user          figure out the owner of the current VT;\n"
+	         "  -s, --switch        switch to the new VT;\n"
+	         "  -w, --wait          wait for command to complete;\n"
+	         "  -v, --verbose       print a message for each action;\n"
+	         "  -V, --version       print program version and exit;\n"
+	         "  -h, --help          output a brief help message.\n"
+	         "\n"),
+	       get_progname());
 	exit(ret);
 }
 
@@ -137,7 +139,7 @@ authenticate_user(int curvt)
 
 	kbd_error(EXIT_FAILURE, 0, _("Couldn't find owner of current tty!"));
 
- got_a_process:
+got_a_process:
 	closedir(dp);
 
 	return pwnam->pw_name;
@@ -159,56 +161,54 @@ open_vt(char *vtname, int force)
 	return fd;
 }
 
-int
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
-	int opt, pid, i;
+	int opt, i;
 	struct vt_stat vtstat;
-	int vtno = -1;
-	int fd  = -1;
-	int consfd = -1;
-	int force = 0;
-	char optc = FALSE;
-	char show = FALSE;
-	char login = FALSE;
-	char verbose = FALSE;
+	int pid          = 0;
+	int vtno         = -1;
+	int fd           = -1;
+	int consfd       = -1;
+	int force        = 0;
+	char optc        = FALSE;
+	char show        = FALSE;
+	char login       = FALSE;
+	char verbose     = FALSE;
 	char direct_exec = FALSE;
-	char do_wait = FALSE;
-	char as_user = FALSE;
-	char vtname[sizeof(VTNAME) + 2]; /* allow 999 possible VTs */
+	char do_wait     = FALSE;
+	char as_user     = FALSE;
+	char vtname[PATH_MAX+1];
 	char *cmd = NULL, *def_cmd = NULL, *username = NULL;
 
 	struct option long_options[] = {
-		{ "help",	no_argument,		0, 'h' },
-		{ "version",	no_argument,		0, 'V' },
-		{ "verbose",	no_argument,		0, 'v' },
-		{ "exec",	no_argument,		0, 'e' },
-		{ "force",	no_argument,		0, 'f' },
-		{ "login",	no_argument,		0, 'l' },
-		{ "user",	no_argument,		0, 'u' },
-		{ "switch",	no_argument,		0, 's' },
-		{ "wait",	no_argument,		0, 'w' },
-		{ "console",	required_argument,	0, 'c' },
+		{ "help", no_argument, 0, 'h' },
+		{ "version", no_argument, 0, 'V' },
+		{ "verbose", no_argument, 0, 'v' },
+		{ "exec", no_argument, 0, 'e' },
+		{ "force", no_argument, 0, 'f' },
+		{ "login", no_argument, 0, 'l' },
+		{ "user", no_argument, 0, 'u' },
+		{ "switch", no_argument, 0, 's' },
+		{ "wait", no_argument, 0, 'w' },
+		{ "console", required_argument, 0, 'c' },
 		{ 0, 0, 0, 0 }
 	};
 
 	set_progname(argv[0]);
-
-	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
-	textdomain(PACKAGE_NAME);
+	setuplocale();
 
 	while ((opt = getopt_long(argc, argv, "c:lsfuewhvV", long_options, NULL)) != -1) {
 		switch (opt) {
 			case 'c':
-				optc = 1;	/* vtno was specified by the user */
+				optc = 1; /* vtno was specified by the user */
 				vtno = (int)atol(optarg);
 
 				if (vtno <= 0 || vtno > 63)
 					kbd_error(5, 0, _("%s: Illegal vt number"), optarg);
 
 				/* close security holes - until we can do this safely */
-				(void)setuid(getuid());
+				if (setuid(getuid()) < 0)
+					kbd_error(5, errno, "%s: setuid", optarg);
 				break;
 			case 'l':
 				login = TRUE;
@@ -265,11 +265,11 @@ main(int argc, char *argv[])
 	} else if (!force) {
 		if (vtno >= 16)
 			kbd_error(7, 0, _("Cannot check whether vt %d is free; use `%s -f' to force."),
-			             vtno, progname);
+			          vtno, get_progname());
 
 		if (vtstat.v_state & (1 << vtno))
 			kbd_error(7, 0, _("vt %d is in use; command aborted; use `%s -f' to force."),
-			             vtno, progname);
+			          vtno, get_progname());
 	}
 
 	if (as_user)
@@ -301,19 +301,19 @@ main(int argc, char *argv[])
 	if (direct_exec || ((pid = fork()) == 0)) {
 		/* leave current vt */
 		if (!direct_exec) {
-#ifdef   ESIX_5_3_2_D
+#ifdef ESIX_5_3_2_D
 #ifdef HAVE_SETPGRP
 			if (setpgrp() < 0)
 #else
 			if (1)
-#endif				/* HAVE_SETPGRP */
+#endif /* HAVE_SETPGRP */
 #else
 			if (setsid() < 0)
-#endif				/* ESIX_5_3_2_D */
+#endif /* ESIX_5_3_2_D */
 				kbd_error(5, errno, _("Unable to set new session"));
 		}
 
-		sprintf(vtname, VTNAME, vtno);
+		snprintf(vtname, PATH_MAX, VTNAME, vtno);
 
 		/* Can we open the vt we want? */
 		if ((fd = open_vt(vtname, force)) == -1) {
@@ -325,18 +325,18 @@ main(int argc, char *argv[])
 				   before giving up. Note: the 16 is a kernel limitation. */
 				for (i = vtno + 1; i < 16; i++) {
 					if ((vtstat.v_state & (1 << i)) == 0) {
-						sprintf(vtname, VTNAME, i);
+						snprintf(vtname, PATH_MAX, VTNAME, i);
 						if ((fd = open_vt(vtname, force)) >= 0) {
 							vtno = i;
 							goto got_vtno;
 						}
 					}
 				}
-				sprintf(vtname, VTNAME, vtno);
+				snprintf(vtname, PATH_MAX, VTNAME, vtno);
 			}
 			kbd_error(5, errsv, _("Unable to open %s"), vtname);
 		}
- got_vtno:
+	got_vtno:
 		if (verbose)
 			kbd_warning(0, _("Using VT %s"), vtname);
 
@@ -350,7 +350,8 @@ main(int argc, char *argv[])
 			uid_t uid = getuid();
 			if (chown(vtname, uid, getgid()) == -1)
 				kbd_error(5, errno, "chown");
-			setuid(uid);
+			if (setuid(uid) < 0)
+				kbd_error(5, errno, "setuid");
 		}
 
 		if (show) {
@@ -389,7 +390,7 @@ main(int argc, char *argv[])
 		wait(NULL);
 		waitpid(pid, &retval, 0);
 
-		if (show) {	/* Switch back... */
+		if (show) { /* Switch back... */
 			if (ioctl(consfd, VT_ACTIVATE, vtstat.v_active))
 				kbd_error(8, errno, "ioctl(VT_ACTIVATE)");
 
@@ -403,7 +404,7 @@ main(int argc, char *argv[])
 
 		/* if all our stuff went right, we want to return the exit code of the command we ran
 		   super vital for scripting loops etc */
-		return(retval);
+		return (retval);
 	}
 
 	return EXIT_SUCCESS;
